# -*- coding: utf-8 -*-
# 샘플 Python 스크립트입니다.

# ⌃R을(를) 눌러 실행하거나 내 코드로 바꿉니다.
# 클래스, 파일, 도구 창, 액션 및 설정을 어디서나 검색하려면 ⇧ 두 번을(를) 누릅니다.

import os
import math

from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *

from PyQt5.QtGui import QPainter, QPen, QFont
from PyQt5.QtWidgets import QAbstractButton, QSlider, QWidget, QVBoxLayout, QHBoxLayout, QStyleOptionSlider, QStyle
from PyQt5.QtCore import Qt, QRect, QPoint

from datetime import datetime
from pprint import pprint


def print_hi(name):
    # 스크립트를 디버그하려면 하단 코드 줄의 중단점을 사용합니다.
    print(f'Hi, {name}')  # 중단점을 전환하려면 ⌘F8을(를) 누릅니다.


class CGravityBar(QGraphicsView):

    def __init__(self, *args):

        self.ratio = None
        self.sceneWidth, self.sceneHeight = None, None
        self.minRange, self.maxRange = -10, 15

        self.scene = QGraphicsScene()
        super().__init__(self.scene)
        self.setScene(self.scene)
        self.setSceneRect(0, 0, 100, 1000)
        self.scene.setBackgroundBrush(Qt.white)
        self.scene.clear()

        # QGraphicsView setting
        self.setFixedWidth(50)
        self.setMinimumHeight(100)

        # self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        # self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.setupUI()
        self.updateTicks(0)

    def setupUI(self):
        self.sceneWidth, self.sceneHeight = 100, 1000
        self.minRange, self.maxRange = -10, 15
        self.ratio = 100
        return

    def setSceneSize(self, width=100, height=1000):
        self.sceneWidth, self.sceneHeight = width, height
        return self.sceneWidth, self.sceneHeight

    def resizeEvent(self, e):
        sceneRect = self.scene.sceneRect()
        self.fitInView(sceneRect)

        return

    def updateTicks(self, value: int):

        ratio = self.ratio

        sceneWidth, sceneHeight = self.sceneWidth, self.sceneHeight
        self.scene.clear()
        outLinePolygon = QPolygonF([
            QPointF(sceneWidth, 0),
            QPointF(sceneWidth, sceneHeight),
            QPointF(sceneWidth // 2, sceneHeight),

            QPointF(0, sceneHeight - sceneWidth // 2),
            QPointF(0, sceneWidth // 2),
            QPointF(sceneWidth // 2, 0)

        ])
        self.scene.addPolygon(outLinePolygon, QPen(Qt.red, 1), QBrush(Qt.white))

        division =3
        #  앞뒤 guard interval=10 을 줌
        datumPoint = sceneHeight - sceneHeight // division

        rangeValue = int(3.0 * ratio)
        intervalValue = sceneHeight // rangeValue

        textFontSize = 15

        #### Center Tick 을 먼저 그린 후에 좌우 눈금을 그린다.
        if value % 5:
            # self.scene.addLine(sceneWidth, datumPoint, (sceneWidth - 20), datumPoint, QPen(Qt.red, 1))
            ...
        elif value % 10:
            self.scene.addLine(sceneWidth, datumPoint, (sceneWidth - 30), datumPoint, QPen(Qt.red, 1))
        else:
            self.scene.addLine(sceneWidth, datumPoint, (sceneWidth - 40), datumPoint, QPen(Qt.blue, 1))
            if not value % 50:
                info = self.scene.addText(str(value / ratio))
                outRect = info.boundingRect()
                info.setPos(QPointF((sceneWidth - 60) - outRect.width(), datumPoint - outRect.height() / 1.5))
                info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

        ### 중심을 기준으로 위쪽을 그린다. 2/3를 그린다.
        for i in range(0, rangeValue - rangeValue // division):
            x_loc = (i + 1) * intervalValue
            valuePoint = value + (i + 1)
            if valuePoint % 5:  # 위쪽 눈금을 그린다.
                # self.scene.addLine(sceneWidth, datumPoint - x_loc, sceneWidth - 20, datumPoint - x_loc, QPen(Qt.red, 1))
                ...
            elif valuePoint % 10:  # 위쪽 눈금을 그린다.
                self.scene.addLine(sceneWidth, datumPoint - x_loc, sceneWidth - 30, datumPoint - x_loc,
                                   QPen(Qt.red, 1))
            else:
                self.scene.addLine(sceneWidth, datumPoint - x_loc, sceneWidth - 40, datumPoint - x_loc,
                                   QPen(Qt.blue, 1))
                if not valuePoint % 50:
                    info = self.scene.addText(str(valuePoint / ratio))
                    outRect = info.boundingRect()
                    info.setPos(
                        QPointF((sceneWidth - 60) - outRect.width(), (datumPoint - x_loc) - outRect.height() / 1.5))
                    info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                    # info.setDefaultTextColor(Qt.white)
                    # info.hide()

        ### 중심을 기준으로 아래 쪽을 그린다. 1/3을 그린다.
        for i in range(1, rangeValue // division):
            x_loc = i * intervalValue

            valuePoint = value - i
            if valuePoint % 5:  # 아래측 눈금을 그린다.
                # self.scene.addLine(sceneWidth, datumPoint + x_loc, sceneWidth - 20, datumPoint + x_loc, QPen(Qt.red, 1))
                ...
            elif valuePoint % 10:  # 아래측 눈금을 그린다.
                self.scene.addLine(sceneWidth, datumPoint + x_loc, sceneWidth - 30, datumPoint + x_loc,
                                   QPen(Qt.red, 1))
            else:
                self.scene.addLine(sceneWidth, datumPoint + x_loc, sceneWidth - 40, datumPoint + x_loc,
                                   QPen(Qt.blue, 1))

                if not valuePoint % 50:
                    info = self.scene.addText(str(valuePoint / ratio))
                    outRect = info.boundingRect()
                    info.setPos(QPointF(sceneWidth - 60 - outRect.width(), datumPoint + x_loc - outRect.height() / 1.5))

                    info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                    # info.setDefaultTextColor(Qt.white)
                    # info.hide()

        # Draw Triangle on Center Point
        bottomLength = 15
        bottomCenterPoint = sceneHeight - sceneHeight // division
        tri_height = int(bottomLength * math.tan(math.radians(60)) * 0.75)
        triangle_polygon = QPolygonF([
            QPointF(sceneWidth, bottomCenterPoint - bottomLength),
            QPointF(sceneWidth - tri_height, bottomCenterPoint),
            QPointF(sceneWidth, bottomCenterPoint + bottomLength)
        ])
        self.scene.addPolygon(triangle_polygon, QPen(Qt.red, 1), QBrush(Qt.red))


class CSpeedBar(QGraphicsView):

    def __init__(self, *args):
        self.enKnotes = None
        self.sceneWidth, self.sceneHeight = None, None

        self.scene = QGraphicsScene()
        super().__init__(self.scene)
        self.setScene(self.scene)
        self.setSceneRect(0, 0, 100, 1000)
        self.scene.setBackgroundBrush(Qt.white)
        self.scene.clear()

        # QGraphicsView setting
        self.setFixedWidth(70)
        self.setMinimumHeight(100)

        # self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        # self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.setupUI()
        self.updateTicks(0)

    def setupUI(self):
        self.ratio =1
        self.enKnotes = True
        self.sceneWidth, self.sceneHeight = 100, 1000
        return

    def setSceneSize(self, width=100, height=1000):
        self.sceneWidth, self.sceneHeight = width, height

    def calcKph2Kn(self, kph):
        return int(round(kph * 0.5399570136727677))

    def calcKn2Kph(self, kn):
        return int(round(kn * 1.851999278976))

    def resizeEvent(self, e):
        sceneRect = self.scene.sceneRect()
        self.fitInView(sceneRect)

        return

    def updateTicks(self, value: int):
        value = self.calcKph2Kn(value) if self.enKnotes else value

        ratio = self.ratio

        sceneWidth, sceneHeight = self.sceneWidth, self.sceneHeight
        self.scene.clear()
        self.scene.addRect(0, 0, sceneWidth, sceneHeight, QPen(Qt.white, 1))

        rangeValue = 100
        intervalValue = sceneHeight // rangeValue
        centerPoint = sceneHeight // 2
        intervalStep = 5 if rangeValue < 100 else 10

        textFontSize = 15

        #### Center Tick 을 먼저 그린 후에 좌우 눈금을 그린다.
        if value % intervalStep:
            self.scene.addLine(sceneWidth, centerPoint, sceneWidth - 25, centerPoint, QPen(Qt.red, 1))
        else:
            self.scene.addLine(sceneWidth, centerPoint, sceneWidth - 40, centerPoint, QPen(Qt.blue, 1))
            info = self.scene.addText(str(value/ratio))
            outRect = info.boundingRect()
            info.setPos(QPointF(sceneWidth - 60 - outRect.width(), centerPoint - outRect.height() / 1.5))
            info.setFont(QFont(info.font().lastResortFont(), textFontSize))

        ### 중심을 기준으로 좌/우측을 동시에 그린다.
        for i in range(0, rangeValue // 2):
            x_loc = (i+1) * intervalValue

            valuePoint = value - (i+1)
            if valuePoint % intervalStep:  # 우측 눈금을 그린다.
                self.scene.addLine(sceneWidth, centerPoint + x_loc, sceneWidth - 25, centerPoint + x_loc,
                                   QPen(Qt.red, 1))
            else:
                self.scene.addLine(sceneWidth, centerPoint + x_loc, sceneWidth - 40, centerPoint + x_loc,
                                   QPen(Qt.blue, 1))
                info = self.scene.addText(str(valuePoint/ratio))
                outRect = info.boundingRect()
                info.setPos(QPointF(sceneWidth - 60 - outRect.width(), centerPoint + x_loc - outRect.height() / 1.5))

                info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

            valuePoint = value + (i+1)
            if valuePoint % intervalStep:  # 우측 눈금을 그린다.
                self.scene.addLine(sceneWidth, centerPoint - x_loc, sceneWidth - 25, centerPoint - x_loc,
                                   QPen(Qt.red, 1))
            else:
                self.scene.addLine(sceneWidth, centerPoint - x_loc, sceneWidth - 40, centerPoint - x_loc,
                                   QPen(Qt.blue, 1))
                info = self.scene.addText(str(valuePoint/ratio))
                outRect = info.boundingRect()
                info.setPos(QPointF(sceneWidth - 60 - outRect.width(), centerPoint - x_loc - outRect.height() / 1.5))
                info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

        # Draw Triangle on Center Point
        bottomLength = 15
        bottomCenterPoint = sceneHeight // 2
        tri_height = int(bottomLength * math.tan(math.radians(60)) * 0.75)
        triangle_polygon = QPolygonF([
            QPointF(sceneWidth, bottomCenterPoint - bottomLength),
            QPointF(sceneWidth - tri_height, bottomCenterPoint),
            QPointF(sceneWidth, bottomCenterPoint + bottomLength)
        ])
        self.scene.addPolygon(triangle_polygon, QPen(Qt.red, 1), QBrush(Qt.red))


class CCompassDisplay(QDial, QWidget):
    signal = pyqtSignal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args)

        self._angle = 0.0
        self._margins = 30
        self._pointText = {0: "N", 45: "NE", 90: "E", 135: "SE", 180: "S", 225: "SW", 270: "W", 315: "NW"}
        self.rotated_base = kwargs.get("rotated", False)
        self.setupUI()

    def setupUI(self):
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setWrapping(True)
        self.setNotchesVisible(True)
        self.setRange(0, 359)
        self.setValue(180)
        # self.valueChanged.connect(self.dial_ValueChanged)
        # self.hide()

        # self.resize(QSize(300, 300))
        # self.setMinimumSize(QSize(100, 100))
        return

    def sizeHint(self):
        return QSize(400, 400)

    # def sizeHint(self):  # 위젯이 가지는 이상적인 크기
    #     return QSize(150, 150)

    def paintEvent(self, paintEvent):
        # print(self.size())
        painter = QPainter()
        painter.begin(self)
        # painter.fillRect(paintEvent.rect(), self.palette().brush(QPalette.Window))

        self.drawMarkings(painter)
        self.drawNeedle(painter)
        painter.end()
        return

    def drawMarkings(self, painter):
        painter.save()
        painter.translate(self.width() / 2, self.height() / 2)
        if self.rotated_base:
            painter.rotate(self._angle)
        scale = min((self.width() - self._margins) / 120.0, (self.height() - self._margins) / 120.0)
        painter.scale(scale, scale)
        font = QFont(self.font())
        font.setPixelSize(6)
        metrics = QFontMetricsF(font)
        painter.setFont(font)
        painter.setPen(self.palette().color(QPalette.Shadow))
        painter.drawEllipse(-45, -45, 90, 90)

        i = 0
        while i < 360:
            if i % 45 == 0:
                painter.drawLine(0, -45, 0, -55)
                painter.drawText(int(metrics.width(self._pointText[i]) * -1 / 2.0), -57, self._pointText[i])
            else:
                painter.drawLine(0, -50, 0, -55)
                point = QPoint(int(metrics.width(str(i)) * -1 / 2.0), -57)
                painter.drawText(point, str(i))

            painter.rotate(15)
            i += 15

        painter.restore()
        return

    def drawNeedle(self, painter):
        painter.save()
        painter.translate(self.width() / 2, self.height() / 2)
        if not self.rotated_base:
            painter.rotate(self._angle)
        scale = min((self.width() - self._margins) / 120.0, (self.height() - self._margins) / 120.0)
        painter.scale(scale, scale)
        painter.setPen(QPen(Qt.NoPen))
        painter.setBrush(self.palette().brush(QPalette.Shadow))
        painter.drawPolygon(QPolygon([QPoint(-8, 0), QPoint(0, -45), QPoint(8, 0), QPoint(0, 45), QPoint(-8, 0)]))
        # painter.setBrush(self.palette().color(QPalette.Mid))
        painter.setBrush(QColor(255, 0, 0))
        painter.drawPolygon(
            QPolygon([QPoint(-5, -17), QPoint(0, -45), QPoint(5, -17), QPoint(0, -20), QPoint(-5, -17)]))

        painter.setPen(QColor(255, 255, 0))
        painter.drawEllipse(-3, -3, 6, 6)

        painter.restore()

    def setAngle(self, angle):
        if angle != self._angle:
            self._angle = angle
        self.update()
        return

    def dial_ValueChanged(self):
        angle = (self.value() + 180) % 360
        print("self.dial.value() = %d, %d" % (angle, (angle + 180) if angle < 180 else (angle - 180)))
        self.setAngle(angle + 180 if angle < 180 else angle - 180)
        return


class CAltitudeBar(QGraphicsView):

    def __init__(self, *args):
        self.enFeet = None
        self.sceneWidth, self.sceneHeight = None, None

        self.scene = QGraphicsScene()
        super().__init__(self.scene)
        self.setScene(self.scene)
        self.setSceneRect(0, 0, 100, 1000)
        self.scene.setBackgroundBrush(Qt.white)
        self.scene.clear()

        # QGraphicsView setting
        self.setFixedWidth(70)
        self.setMinimumHeight(100)

        # self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        # self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.setupUI()
        self.updateTicks(0)

    def setupUI(self):
        self.ratio =1
        self.enFeet = True
        self.sceneWidth, self.sceneHeight = 100, 1000
        return

    def setSceneSize(self, width=100, height=1000):
        self.sceneWidth, self.sceneHeight = width, height

    def calcMeter2Ft(self, m):
        return int(round(m * 3.28084))

    def calcFt2Meter(self, ft):
        return int(round(ft * 0.3048))

    def resizeEvent(self, e):
        sceneRect = self.scene.sceneRect()
        self.fitInView(sceneRect)

        return

    def updateTicks(self, value: int):
        value = self.calcMeter2Ft(value) if self.enFeet else value
        ratio = self.ratio
        sceneWidth, sceneHeight = self.sceneWidth, self.sceneHeight
        self.scene.clear()
        self.scene.addRect(0, 0, sceneWidth, sceneHeight, QPen(Qt.white, 1))

        rangeValue = 100
        intervalValue = sceneHeight // rangeValue
        centerPoint = sceneHeight // 2
        intervalStep = 5 if rangeValue < 100 else 10

        textFontSize = 15

        #### Center Tick 을 먼저 그린 후에 좌우 눈금을 그린다.
        if value % intervalStep:
            self.scene.addLine(0, centerPoint, 25, centerPoint, QPen(Qt.red, 1))
        else:
            self.scene.addLine(0, centerPoint, 40, centerPoint, QPen(Qt.blue, 1))
            info = self.scene.addText(str(value/ratio))
            outRect = info.boundingRect()
            info.setPos(QPointF(75 - outRect.width(), centerPoint - outRect.height() / 1.5))
            info.setFont(QFont(info.font().lastResortFont(), textFontSize))

        ### 중심을 기준으로 좌/우측을 동시에 그린다.
        for i in range(0, rangeValue // 2):
            x_loc = (i+1) * intervalValue

            valuePoint = value - (i+1)
            if valuePoint % intervalStep:  # 우측 눈금을 그린다.
                self.scene.addLine(0, centerPoint + x_loc, 25, centerPoint + x_loc, QPen(Qt.red, 1))
            else:
                self.scene.addLine(0, centerPoint + x_loc, 40, centerPoint + x_loc, QPen(Qt.blue, 1))
                info = self.scene.addText(str(valuePoint/ratio))
                outRect = info.boundingRect()
                info.setPos(QPointF(75 - outRect.width(), centerPoint + x_loc - outRect.height() / 1.5))

                info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

            valuePoint = value + (i+1)
            if valuePoint % intervalStep:  # 우측 눈금을 그린다.
                self.scene.addLine(0, centerPoint - x_loc, 25, centerPoint - x_loc, QPen(Qt.red, 1))
            else:
                self.scene.addLine(0, centerPoint - x_loc, 40, centerPoint - x_loc, QPen(Qt.blue, 1))
                info = self.scene.addText(str(valuePoint/ratio))
                outRect = info.boundingRect()
                info.setPos(QPointF(75 - outRect.width(), centerPoint - x_loc - outRect.height() / 1.5))
                info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

        # Draw Triangle on Center Point
        bottomLength = 15
        bottomCenterPoint = sceneHeight // 2
        tri_height = int(bottomLength * math.tan(math.radians(60)) * 0.75)
        triangle_polygon = QPolygonF([
            QPointF(0, bottomCenterPoint - bottomLength),
            QPointF(tri_height, bottomCenterPoint),
            QPointF(0, bottomCenterPoint + bottomLength)
        ])
        self.scene.addPolygon(triangle_polygon, QPen(Qt.red, 1), QBrush(Qt.red))
        return

class CVerticalSpeedBar(QGraphicsView):

    def __init__(self, *args):

        self.ratio = None
        self.sceneWidth, self.sceneHeight = None, None
        self.minRange, self.maxRange = -10, 15

        self.scene = QGraphicsScene()
        super().__init__(self.scene)
        self.setScene(self.scene)
        self.setSceneRect(0, 0, 100, 1000)
        self.scene.setBackgroundBrush(Qt.white)
        self.scene.clear()

        # QGraphicsView setting
        self.setFixedWidth(50)
        self.setMinimumHeight(100)

        # self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        # self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.setupUI()
        self.updateTicks(0)

    def setupUI(self):
        self.sceneWidth, self.sceneHeight = 100, 1000
        self.minRange, self.maxRange = -10, 15
        self.ratio = 100
        return

    def setSceneSize(self, width=100, height=1000):
        self.sceneWidth, self.sceneHeight = width, height
        return self.sceneWidth, self.sceneHeight

    def resizeEvent(self, e):
        sceneRect = self.scene.sceneRect()
        self.fitInView(sceneRect)

        return

    def updateTicks(self, value: int):

        ratio = self.ratio

        sceneWidth, sceneHeight = self.sceneWidth, self.sceneHeight
        self.scene.clear()
        outLinePolygon = QPolygonF([
            QPointF(0, 0),
            QPointF(0, sceneHeight),
            QPointF(sceneWidth // 2, sceneHeight),
            QPointF(sceneWidth, sceneHeight - sceneWidth // 2),
            QPointF(sceneWidth, sceneWidth // 2),
            QPointF(sceneWidth // 2, 0),
        ])
        self.scene.addPolygon(outLinePolygon, QPen(Qt.red, 1), QBrush(Qt.white))

        #  앞뒤 guard interval=10 을 줌
        datumPoint = 20 + sceneHeight // 3

        rangeValue = int(3.0 * ratio)
        intervalValue = (sceneHeight - 20) // rangeValue

        textFontSize = 15

        #### Center Tick 을 먼저 그린 후에 좌우 눈금을 그린다.
        if value % 5:
            # self.scene.addLine(0, datumPoint, 20, datumPoint, QPen(Qt.red, 1))
            ...
        elif value % 10:
            self.scene.addLine(0, datumPoint, 30, datumPoint, QPen(Qt.red, 1))
        else:
            self.scene.addLine(0, datumPoint, 40, datumPoint, QPen(Qt.blue, 1))
            if not value % 50:
                info = self.scene.addText(str(value / ratio))
                outRect = info.boundingRect()
                info.setPos(QPointF(70 - outRect.width(), datumPoint - outRect.height() / 1.5))
                info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

        ### 중심을 기준으로 위쪽을 그린다. 2/3를 그린다.
        for i in range(0, rangeValue // 3):
            x_loc = (i + 1) * intervalValue
            valuePoint = value + (i + 1)
            if valuePoint % 5:  # 위쪽 눈금을 그린다.
                # self.scene.addLine(0, datumPoint - x_loc, 20, datumPoint - x_loc, QPen(Qt.red, 1))
                ...
            elif valuePoint % 10:  # 위쪽 눈금을 그린다.
                self.scene.addLine(0, datumPoint - x_loc, 30, datumPoint - x_loc,
                                   QPen(Qt.red, 1))
            else:
                self.scene.addLine(0, datumPoint - x_loc, 40, datumPoint - x_loc,
                                   QPen(Qt.blue, 1))
                if not valuePoint % 50:
                    info = self.scene.addText(str(valuePoint / ratio))
                    outRect = info.boundingRect()
                    info.setPos(
                        QPointF(70 - outRect.width(), (datumPoint - x_loc) - outRect.height() / 1.5))
                    info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                    # info.setDefaultTextColor(Qt.white)
                    # info.hide()

        ### 중심을 기준으로 아래 쪽을 그린다. 1/3을 그린다.
        for i in range(1, rangeValue - rangeValue // 3):
            x_loc = i * intervalValue

            valuePoint = value - i
            if valuePoint % 5:  # 아래측 눈금을 그린다.
                # self.scene.addLine(0, datumPoint + x_loc, 20, datumPoint + x_loc, QPen(Qt.red, 1))
                ...
            elif valuePoint % 10:  # 아래측 눈금을 그린다.
                self.scene.addLine(0, datumPoint + x_loc, 30, datumPoint + x_loc,
                                   QPen(Qt.red, 1))
            else:
                self.scene.addLine(0, datumPoint + x_loc, 40, datumPoint + x_loc,
                                   QPen(Qt.blue, 1))

                if not valuePoint % 50:
                    info = self.scene.addText(str(valuePoint / ratio))
                    outRect = info.boundingRect()
                    info.setPos(QPointF(70 - outRect.width(), datumPoint + x_loc - outRect.height() / 1.5))

                    info.setFont(QFont(info.font().lastResortFont(), textFontSize))
                    # info.setDefaultTextColor(Qt.white)
                    # info.hide()

        # Draw Triangle on Center Point
        bottomLength = 15
        bottomCenterPoint = 20 + sceneHeight // 3
        tri_height = int(bottomLength * math.tan(math.radians(60)) * 0.75)
        triangle_polygon = QPolygonF([
            QPointF(0, bottomCenterPoint - bottomLength),
            QPointF(tri_height, bottomCenterPoint),
            QPointF(0, bottomCenterPoint + bottomLength)
        ])
        self.scene.addPolygon(triangle_polygon, QPen(Qt.red, 1), QBrush(Qt.red))

class CCompassBar(QGraphicsView):

    def __init__(self, *args):
        self.scene = QGraphicsScene()
        super().__init__(self.scene)
        self.setScene(self.scene)
        self.setSceneRect(0, 0, 1000, 100)
        self.scene.setBackgroundBrush(Qt.white)
        self.scene.clear()

        # use in draw sequence
        # - items : Scene 에 추가된 item list
        # - 임시 item 들은 obj에 관리된다.
        self.items = []
        self.obj = {}  # View 출력을 위한 임시 저장장소

        # QGraphicsView setting
        # self.setMinimumHeight(10)
        self.setFixedHeight(50)
        self.setMinimumWidth(100)

        self.updateTicks(0)

    def resizeEvent(self, e):
        sceneRect = self.scene.sceneRect()
        self.fitInView(sceneRect)

        return

    def updateTicks(self, angle: int):
        angle %= 360
        ratio = 0.6

        sceneWidth, sceneHeight = 1000, 100
        self.scene.clear()
        self.scene.addRect(0, 0, sceneWidth, sceneHeight, QPen(Qt.white, 1))

        rangeValue = 120
        intervalValue = sceneWidth // rangeValue
        centerPoint = sceneWidth // 2
        intervalStep = 5 if rangeValue < 100 else 10

        #### Center Tick 을 먼저 그린 후에 좌우 눈금을 그린다.
        if angle % intervalStep:
            self.scene.addLine(centerPoint, 40, centerPoint, 60, QPen(Qt.red, 1))
        else:
            self.scene.addLine(centerPoint, 30, centerPoint, 70, QPen(Qt.blue, 1))
            info = self.scene.addText(str(angle))
            outRect = info.boundingRect()
            info.setPos(QPointF(centerPoint - outRect.width() + 3, 80 - outRect.height() / 3))
            info.setFont(QFont(info.font().lastResortFont(), 20))
            # info.setDefaultTextColor(Qt.white)
            # info.hide()

        ### 중심을 기준으로 좌/우측을 동시에 그린다.
        for i in range(0, rangeValue // 2):
            x_loc = (i + 1) * intervalValue

            anglePoint = angle + (i + 1)
            if anglePoint % 5:  # 우측 눈금을 그린다.
                # self.scene.addLine(centerPoint + x_loc, 40, centerPoint + x_loc, 60, QPen(Qt.red, 1))
                ...
            elif anglePoint % 10:
                self.scene.addLine(centerPoint + x_loc, 40, centerPoint + x_loc, 60, QPen(Qt.red, 1))
                if anglePoint % 360 == 45:
                    txt = "NE"
                elif anglePoint % 360 == 135:
                    txt = "ES"
                elif anglePoint % 360 == 225:
                    txt = "SW"
                elif anglePoint % 360 == 315:
                    txt = "WN"
                else:
                    txt = ""
                if txt:
                    info = self.scene.addText(txt)
                    outRect = info.boundingRect()
                    info.setPos(QPointF(centerPoint + x_loc - outRect.width() / 1.2, 0))
                    info.setFont(QFont(info.font().lastResortFont(), 20))
            else:
                self.scene.addLine(centerPoint + x_loc, 30, centerPoint + x_loc, 70, QPen(Qt.blue, 1))
                info = self.scene.addText(str(anglePoint % 360))
                outRect = info.boundingRect()
                info.setPos(QPointF(centerPoint + x_loc - outRect.width() + 3, 80 - outRect.height() / 3))
                info.setFont(QFont(info.font().lastResortFont(), 20))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

                if anglePoint % 360 == 0:
                    txt = "N"
                elif anglePoint % 360 == 90:
                    txt = "E"
                elif anglePoint % 360 == 180:
                    txt = "S"
                elif anglePoint % 360 == 270:
                    txt = "W"
                else:
                    txt = ""
                if txt:
                    info = self.scene.addText(txt)
                    outRect = info.boundingRect()
                    info.setPos(QPointF(centerPoint + x_loc - outRect.width() / 1.2, 0))
                    info.setFont(QFont(info.font().lastResortFont(), 20))

            anglePoint = angle - (i + 1)
            if anglePoint % 5:  # 좌측 눈금을 그린다.
                # self.scene.addLine(centerPoint - x_loc, 40, centerPoint - x_loc, 60, QPen(Qt.red, 1))
                ...
            elif anglePoint % 10:
                self.scene.addLine(centerPoint - x_loc, 40, centerPoint - x_loc, 60, QPen(Qt.red, 1))
                if anglePoint % 360 == 45:
                    txt = "NE"
                elif anglePoint % 360 == 135:
                    txt = "ES"
                elif anglePoint % 360 == 225:
                    txt = "SW"
                elif anglePoint % 360 == 315:
                    txt = "WN"
                else:
                    txt = ""
                if txt:
                    info = self.scene.addText(txt)
                    outRect = info.boundingRect()
                    info.setPos(QPointF(centerPoint - x_loc - outRect.width() / 1.2 - 5, 0))
                    info.setFont(QFont(info.font().lastResortFont(), 20))
            else:
                self.scene.addLine(centerPoint - x_loc, 30, centerPoint - x_loc, 70, QPen(Qt.blue, 1))
                info = self.scene.addText(str(anglePoint % 360))
                outRect = info.boundingRect()
                info.setPos(QPointF(centerPoint - x_loc - outRect.width() + 3, 80 - outRect.height() / 3))
                info.setFont(QFont(info.font().lastResortFont(), 20))
                # info.setDefaultTextColor(Qt.white)
                # info.hide()

                if anglePoint % 360 == 0:
                    txt = "N"
                elif anglePoint % 360 == 90:
                    txt = "E"
                elif anglePoint % 360 == 180:
                    txt = "S"
                elif anglePoint % 360 == 270:
                    txt = "W"
                else:
                    txt = ""
                if txt:
                    info = self.scene.addText(txt)
                    outRect = info.boundingRect()
                    info.setPos(QPointF(centerPoint - x_loc - outRect.width() / 1.2, 0))
                    info.setFont(QFont(info.font().lastResortFont(), 20))

        # Draw Triangle on Center Point
        bottomLength = 20
        bottomCenterPoint = sceneWidth // 2 - 1
        tri_height = int(bottomLength * math.tan(math.radians(60)) * 0.75)
        triangle_polygon = QPolygonF([
            QPointF(bottomCenterPoint - bottomLength, 0),
            QPointF(bottomCenterPoint, tri_height),
            QPointF(bottomCenterPoint + bottomLength, 0)
        ])

        self.scene.addPolygon(triangle_polygon, QPen(Qt.red, 1), QBrush(Qt.red))

        return


class CPrimaryFlightDisplay(QWidget):
    signal = pyqtSignal(object, name="pyqtSignal")

    def __init__(self, *args):
        super().__init__()

        self.m_gravityBar = None
        self.m_groundSpeedBar = None
        self.m_flightDisplay = None
        self.m_altitudeBar = None
        self.m_verticalSpeedBar = None
        self.m_compassBar = None

        self.spinBox = QSpinBox()
        self.spinBox.setRange(-720, 720)
        self.spinBox.valueChanged.connect(self.setSliceValue)
        self.setupUI()

    def setSliceValue(self):
        val = self.spinBox.value()
        self.m_gravityBar.updateTicks(val)
        self.m_groundSpeedBar.updateTicks(val)
        self.m_altitudeBar.updateTicks(val)
        self.m_verticalSpeedBar.updateTicks(val)
        self.m_compassBar.updateTicks(val)

        return

    def sizeHint(self):
        return QSize(500, 500)

    def setupUI(self):
        centralLayout = QGridLayout()
        self.m_gravityBar = CGravityBar(Qt.Vertical)
        self.m_groundSpeedBar = CSpeedBar()
        self.m_flightDisplay = CCompassDisplay()
        self.m_altitudeBar = CAltitudeBar()
        self.m_verticalSpeedBar = CVerticalSpeedBar()
        self.m_compassBar = CCompassBar()

        centralLayout.addWidget(self.m_gravityBar, 0, 0)
        centralLayout.addWidget(self.m_groundSpeedBar, 0, 1)
        centralLayout.addWidget(self.m_flightDisplay, 0, 2, Qt.AlignCenter)
        centralLayout.addWidget(self.m_altitudeBar, 0, 3)
        centralLayout.addWidget(self.m_verticalSpeedBar, 0, 4)
        centralLayout.addWidget(self.m_compassBar, 1, 1, 1, 3)

        centralLayout.addWidget(self.spinBox, 2, 2, 1, 1)

        self.setLayout(centralLayout)
        # self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        return


# 스크립트를 실행하려면 여백의 녹색 버튼을 누릅니다.
if __name__ == '__main__':
    import sys

    print_hi('PyCharm')
    curPath = os.getcwd()
    workPath = os.path.realpath(__file__)
    os.chdir(os.path.dirname(workPath))
    app = QApplication(sys.argv)
    myWindow = CPrimaryFlightDisplay()
    myWindow.show()
    os.chdir(curPath)
    sys.exit(app.exec_())

# https://www.jetbrains.com/help/pycharm/에서 PyCharm 도움말 참조
